function jsonToDataMatrix(json) {
    var matrix = [];
    var index = [];
    if (!json || json.length === 0)
        return { matrix: matrix, index: index, columns: [] };
    var columns = Object.keys(json[0]);
    json.forEach(function (row, i) {
        matrix.push(Object.values(row));
        index.push(i);
    });
    return { matrix: matrix, index: index, columns: columns };
}
function columnsToDataMatrix(_a) {
    var source = _a.source, cols = _a.columns;
    var matrix = [];
    var index = [];
    if (!source || source.length === 0)
        return { matrix: matrix, index: index, columns: [] };
    var columns = cols || source.map(function (_, i) { return "col_" + i; });
    var rowLength = 0;
    source.forEach(function (column) {
        if (column.length > rowLength) {
            rowLength = column.length;
        }
    });
    for (var i = 0; i < rowLength; i++) {
        matrix.push([]);
        index.push(i);
    }
    source.forEach(function (colData) {
        for (var i = 0; i < rowLength; i++) {
            matrix[i].push(colData[i] || undefined);
        }
    });
    return { matrix: matrix, index: index, columns: columns };
}
function rowsToDataMatrix(_a) {
    var source = _a.source, cols = _a.columns;
    var matrix = [];
    var index = [];
    if (!source || source.length === 0)
        return { matrix: matrix, index: index, columns: [] };
    var columnLength = 0;
    source.forEach(function (row) {
        if (row.length > columnLength) {
            columnLength = row.length;
        }
    });
    var columns = [];
    for (var i = 0; i < columnLength; i++) {
        if (cols && cols[i]) {
            columns.push(cols[i]);
        }
        else {
            columns.push("col_" + i);
        }
    }
    for (var i = 0; i < source.length; i++) {
        matrix.push([]);
        index.push(i);
    }
    source.forEach(function (row, i) {
        for (var j = 0; j < columnLength; j++) {
            matrix[i].push(row[j] || undefined);
        }
    });
    return { matrix: matrix, index: index, columns: columns };
}
/**
 * Adjust structure of data.
 *
 * To avoid missing keys and values.
 */
function adjustJSONData(json, mode) {
    if (mode === void 0) { mode = 'max'; }
    if (!json || json.length === 0)
        return [];
    var resultJSONData = [];
    // consistent fields
    var fieldSet = new Set();
    if (mode === 'max') {
        fieldSet = new Set();
        json.forEach(function (row) {
            Object.keys(row).forEach(function (key) {
                fieldSet.add(key);
            });
        });
    }
    else {
        // mode === 'min
        fieldSet = new Set(Object.keys(json[0]));
        json.forEach(function (row) {
            var keys = Object.keys(row);
            fieldSet.forEach(function (field) {
                if (!keys.includes(field)) {
                    fieldSet.delete(field);
                }
            });
        });
    }
    // adjust data row by row
    json.forEach(function (row) {
        var newRow = {};
        fieldSet.forEach(function (fieldName) {
            newRow[fieldName] = row[fieldName] || undefined;
        });
        resultJSONData.push(newRow);
    });
    return resultJSONData;
}
/**
 * @public
 */
var Dataset = /** @class */ (function () {
    function Dataset(type, params) {
        this.dataMatrix = [];
        this.dataIndex = [];
        this.dataColumns = [];
        switch (type) {
            case 'json': {
                var source = params.source;
                var data = adjustJSONData(source);
                var _a = jsonToDataMatrix(data), matrix = _a.matrix, index = _a.index, columns = _a.columns;
                this.dataMatrix = matrix;
                this.dataIndex = index;
                this.dataColumns = columns;
                break;
            }
            case 'rows': {
                var _b = rowsToDataMatrix(params), matrix = _b.matrix, index = _b.index, columns = _b.columns;
                this.dataMatrix = matrix;
                this.dataIndex = index;
                this.dataColumns = columns;
                break;
            }
            case 'columns': {
                var _c = columnsToDataMatrix(params), matrix = _c.matrix, index = _c.index, columns = _c.columns;
                this.dataMatrix = matrix;
                this.dataIndex = index;
                this.dataColumns = columns;
                break;
            }
            default:
                this.dataMatrix = [];
                this.dataIndex = [];
                this.dataColumns = [];
                break;
        }
    }
    Dataset.prototype.toJson = function () {
        var _this = this;
        return this.dataMatrix.map(function (row) {
            var rowObj = {};
            _this.dataColumns.forEach(function (key, i) {
                rowObj[key] = row[i];
            });
            return rowObj;
        });
    };
    return Dataset;
}());
export { Dataset };
