"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFullCombination = void 0;
/**
 * @public
 */
function isFullCombination(dataset) {
    var data = dataset.toJson();
    // Empty dataset or one record dataset is not full-combination by definition.
    if (!data || data.length <= 1)
        return false;
    var fieldNames = Object.keys(data[0]);
    var OptionalValuesOfFields = {};
    fieldNames.forEach(function (fieldName) {
        OptionalValuesOfFields[fieldName] = new Set();
    });
    var recordStrUUIDs = new Set();
    data.forEach(function (row) {
        var keys = Object.keys(row);
        keys.forEach(function (key) {
            OptionalValuesOfFields[key].add(row[key]);
        });
        var vStr = JSON.stringify(keys.sort().map(function (key) {
            var _a;
            return _a = {}, _a[key] = row[key], _a;
        }));
        recordStrUUIDs.add(vStr);
    });
    // If records have repetition, fail by definition.
    if (recordStrUUIDs.size !== data.length)
        return false;
    var fullCombSize = Object.values(OptionalValuesOfFields)
        .map(function (set) { return set.size; })
        .reduce(function (acc, cur) { return acc * cur; });
    // Succeed only if full-combination size equals to the size of dataset.
    if (fullCombSize === data.length)
        return true;
    return false;
}
exports.isFullCombination = isFullCombination;
