
/**
 * @public
 */
export declare type DataColumnArray = any[];

/**
 * @public
 */
export declare type DataColumnObject = Record<string, any>;

/**
 * @public
 */
export declare type DataRow = DataRowObject | DataRowArray;

/**
 * @public
 */
export declare type DataRowArray = any[];

/**
 * @public
 */
export declare type DataRowObject = Record<string, any>;

/**
 * @public
 */
export declare class Dataset {
    dataMatrix: any[][];
    dataIndex: number[];
    dataColumns: string[];
    constructor(type: 'json', params: DatasetParamsByJson);
    constructor(type: 'rows', params: DatasetParamsByRows);
    constructor(type: 'columns', params: DatasetParamsByColumns);
    toJson(): Record<string, any>[];
}

/**
 * @public
 */
export declare type DatasetParams = DatasetParamsByJson | DatasetParamsByRows | DatasetParamsByColumns;

/**
 * @public
 */
export declare interface DatasetParamsByColumns {
    source: DataColumnArray[];
    columns?: string[];
}

/**
 * @public
 */
export declare interface DatasetParamsByJson {
    source: JSONData;
}

/**
 * @public
 */
export declare interface DatasetParamsByRows {
    source: DataRowArray[];
    columns?: string[];
}

/**
 * @public
 */
export declare function isDataRowArray(row: DataRow): row is DataRowArray;

/**
 * @public
 */
export declare function isDataRowObject(row: DataRow): row is DataRowObject;

/**
 * @public
 */
export declare function isFullCombination(dataset: Dataset): boolean;

/**
 * @public
 */
export declare function isInHierarchy(parent: DataColumnArray, child: DataColumnArray): boolean;

/**
 * @public
 */
export declare type JSONData = DataRowObject[];

/**
 * @public
 */
export declare type SourceType = 'json' | 'rows' | 'columns';

export { }
